#!/bin/sh
#-----------------------------------------------------------------------------
# /tmp/preinstall.sh  - pre-install script for OpenLDAP  server
#
# Copyright (c) 2004-2008  Jens Vehlhaber <jvehlhaber(at)buchenwald(dot)de>
# Copyright (c) 2009-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2005-03-29 jv
# Last Update:  $Id: preinstall.sh 72545 2020-02-02 18:04:51Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-----------------------------------------------------------------------------

#include eislib etc.
. /var/install/include/check-eisfair-version

# directory names
ldap_confdir=/etc/openldap
ldap_datadir=/var/lib/openldap

if [ "${EISFAIR_SYSTEM}" = 'eisfair-64' ]
then
    ldap_sasldir=/usr/lib64/sasl3
else
    ldap_sasldir=/usr/lib/sasl3
fi

# file names
module_name=ldapserver

configfile=/etc/config.d/${module_name}
deinstallfile=/var/install/deinstall/${module_name}
old_configfile=/etc/config.d/openldap
old_deinstallfile=/var/install/deinstall/openldap
installfile=/var/run/${module_name}.install
slapd_configfile=${ldap_confdir}/slapd.conf
sasl_configfile=${ldap_sasldir}/slapd.conf

# extract shared library file from archive
# to be able to use the myecho command
archivefile=`find /tmp -maxdepth 1 -type l -name "${module_name}.tar.*" ! -name "*.info" | head -n1`

if [ -n "${archivefile}" ]
then
    if [ -s "${archivefile}" ]
    then
        tar xjf ${archivefile} var/install/include/jedlib.${module_name} -C /

        srcfile=/var/install/include/jedlib.${module_name}
        destfile=/var/install/include/jedlib

        if [ -f ${srcfile} ]
        then
            if [ -f ${destfile} ]
            then
                srcver=`/bin/sh ${srcfile} version`
                destver=`/bin/sh ${destfile} version`

                if [ -n "${destver}" ]
                then
                    # version number exists - compare version numbers
                    if [ `echo "${srcver} > ${destver}" | bc` -eq 1 ]
                    then
                        # update file
                        echo "updating shared library file (${destver}->${srcver}) ..."
                        mv -v ${srcfile} ${destfile}
                    fi
                else
                    # no version number exists - update file
                    echo "updating shared library file ..."
                    mv -v ${srcfile} ${destfile}
                fi
            else
                # file doesn't exist - update file
                echo "updating shared library file ..."
                mv -v ${srcfile} ${destfile}
            fi
        fi
    fi
fi

# include eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

myecho --pack ${module_name} --info "running preinstall.sh ..."

# required configuration files exist, go on ...
if [ -f ${configfile} -o -f ${old_configfile} ]
then
    # backup existing configuration file
    myecho --pack ${module_name} "saving previous configuration file to ${installfile} ..."

    if [ -f ${old_configfile} ]
    then
        # copy previous openldap to ldapserver configuration because
        # the old configuration file is required to shutdown the old
        # processes
        cp ${old_configfile} ${configfile}

        # renaming previous openldap backup file
        ls /etc/backup.d/openldap.* >/dev/null 2>/dev/null

        if [ $? -eq 0 ]
        then
            myecho --pack ${module_name} "renaming previous backup file(s) ..."

            for FNAME in `ls /etc/backup.d/openldap.* 2>/dev/null`
            do
                newfile=`echo ${FNAME} | sed "s#\/openldap\.#\/${module_name}\.#g"`
                mv $FNAME ${newfile}
            done
        fi
    fi

    /var/install/bin/backup-file --quiet ${configfile}

    # check if admin name exists ...
    grep -q "^LDAPSERVER_LDAP_ADMIN_NAME" ${configfile}

    if [ $? -eq 1 ]
    then
        # not found, try to add it ...
        ldapadmin=`grep "ldapdb_id:" ${sasl_configfile} | cut -d' ' -f2`
        echo "LDAPSERVER_LDAP_ADMIN_NAME='${ldapadmin}'" >> ${configfile}
    fi

    # check if admin password exists ...
    grep -q "^LDAPSERVER_LDAP_ADMIN_PASS" ${configfile}

    if [ $? -eq 1 ]
    then
        # not found, try to add it ...
        ldappass=`grep "ldapdb_pw:" ${sasl_configfile} | cut -d' ' -f2`
        echo "LDAPSERVER_LDAP_ADMIN_PASS='${ldappass}'" >> ${configfile}
    fi

    # save previous configuration file ...
    cp ${configfile} ${installfile}
fi

# update package
if [ -f ${old_deinstallfile} ]
then
    # openldap
    if [ ! -f ${slapd_configfile} ]
    then
        myecho --pack ${module_name} --error "Required file '${slapd_configfile}' doesn't exist, installation aborted!"
        myanykey --pack ${module_name}
        exit 1
    fi

    if [ ! -f ${sasl_configfile} ]
    then
        myecho --pack ${module_name} --error "Required file '${sasl_configfile}' doesn't exist, installation aborted!"
        myanykey --pack ${module_name}
        exit 1
    fi

    # export ldap database
    myecho --pack ${module_name} "exporting ldap tree to /root/openldap-export.ldif ..."

    ldaprootdn=`grep "^rootdn" ${slapd_configfile} | cut -d\" -f2`
    ldappass=`grep "ldapdb_pw:" ${sasl_configfile} | cut -d' ' -f2`

    ldapsearch -x -D ${ldaprootdn} -w "${ldappass}" > /root/openldap-export.ldif

    if [ $? -ne 0 ]
    then
        myecho --pack ${module_name} --force-output --info "Please enter admin password for account '${ldaprootdn}':"
        ldapsearch -x -D ${ldaprootdn} -W > /root/openldap-export.ldif

        if [ $? -eq 0 ]
        then
            myecho --pack ${module_name} --force-output "done."
        else
            # error appeared during tree export
            rm -f /root/openldap-export.ldif
        fi
    fi

    ldap_archive=${module_name}-${EISDATE}-${EISTIME}.tgz

    myecho --pack ${module_name} "saving previous ldap files to /root/${ldap_archive} ..."

    tar czf /tmp/${ldap_archive} /etc/openldap /var/lib/openldap ${ldap_sasldir}/slapd.conf --exclude=*tgz 2>/dev/null
    mv /tmp/${ldap_archive} /root/

    myecho --pack ${module_name} "deinstalling previously installed software ..."
    ${old_deinstallfile} update

    # remove obsolete files
    rm -f /etc/openldap/ldap.conf.default
    rm -f /etc/openldap/slapd.conf.default
    rm -f /var/install/deinstall/openldap
elif [ -f ${deinstallfile} ]
then
    # ldapserver
    myecho --pack ${module_name} "deinstalling previously installed software ..."
    /usr/bin/eisman uninstall --update --auto --quiet ${module_name}
fi

exit 0
