#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/ldapserver-update.sh - ldapserver parameter update script
#
# Copyright (c) 2004-2008  Jens Vehlhaber <jvehlhaber(at)buchenwald(dot)de>
# Copyright (c) 2009-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2005-04-14 jv
# Last Update:  $Id: ldapserver-update.sh 72521 2020-02-02 14:02:30Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/jedlib
. /var/install/include/configlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ! -z "`grep ^OPENLDAP_DO_DEBUG ${source_conf}`" ]
    then
        if [ "${OPENLDAP_DO_DEBUG}" = "yes" ]
        then
            echo "- OPENLDAP_DO_DEBUG -> LDAPSERVER_DO_DEBUG" >> ${_rv_output_file}
            LDAPSERVER_DO_DEBUG="${OPENLDAP_DO_DEBUG}"
            _rv_renamed=1
        fi
    fi

    if [ ! -z "`grep ^START_OPENLDAP ${source_conf}`" ]
    then
        echo "- START_OPENLDAP -> START_LDAPSERVER" >> ${_rv_output_file}
        START_LDAPSERVER="${START_OPENLDAP}"
        _rv_renamed=1
    fi

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file
    local _av_new_val

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # base domain component, e.g. privatnet
    if [ -z "${LDAPSERVER_LDAP_BASEDC}" -a -z "`grep ^LDAPSERVER_LDAP_BASEDC ${source_conf}`" ]
    then
        _av_new_val="`echo ${DOMAIN_NAME} | cut -d. -f1`"
        echo "- LDAPSERVER_LDAP_BASEDC='${_av_new_val}'" >> ${_av_output_file}
        LDAPSERVER_LDAP_BASEDC="${_av_new_val}"
        _av_added=1
    fi

    # base distinguished name, e.g. dc=privatnet,dc=lan
    if [ -z "${LDAPSERVER_LDAP_BASEDN}" -a -z "`grep ^LDAPSERVER_LDAP_BASEDN ${source_conf}`" ]
    then
        _av_new_val="dc=`echo ${DOMAIN_NAME} | sed 's/\./,dc=/g'`"
        echo "- LDAPSERVER_LDAP_BASEDN='${_av_new_val}'" >> ${_av_output_file}
        LDAPSERVER_LDAP_BASEDN="${_av_new_val}"
        _av_added=1
    fi

    # name of LDAP administrator
    if [ -z "${LDAPSERVER_LDAP_ADMIN_NAME}" -a -z "`grep ^LDAPSERVER_LDAP_ADMIN_NAME ${source_conf}`" ]
    then
        echo "- LDAPSERVER_LDAP_ADMIN_NAME='ldapadmin'" >> ${_av_output_file}
        LDAPSERVER_LDAP_ADMIN_NAME='ldapadmin'
        _av_added=1
    fi

    # password of LDAP administrator
    if [ -z "${LDAPSERVER_LDAP_ADMIN_PASS}" -a -z "`grep ^LDAPSERVER_LDAP_ADMIN_PASS ${source_conf}`" ]
    then
        echo "- LDAPSERVER_LDAP_ADMIN_PASS=''" >> ${_av_output_file}
        LDAPSERVER_LDAP_ADMIN_PASS=''
        _av_added=1
    fi

    #--------------------------------------------------------------------------

    if [ -z "${LDAPSERVER_TRANSPORT}" -a -z "`grep ^LDAPSERVER_TRANSPORT ${source_conf}`" ]
    then
        echo "- LDAPSERVER_TRANSPORT='default'" >> ${_av_output_file}
        LDAPSERVER_TRANSPORT='default'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_TLS_PROTOCOL}" -a -z "`grep ^LDAPSERVER_TLS_PROTOCOL ${source_conf}`" ]
    then
        echo "- LDAPSERVER_TLS_PROTOCOL='none'" >> ${_av_output_file}
        LDAPSERVER_TLS_PROTOCOL='none'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_CLIENT_FORCE_TLS}" -a -z "`grep ^LDAPSERVER_CLIENT_FORCE_TLS ${source_conf}`" ]
    then
        echo "- LDAPSERVER_CLIENT_FORCE_TLS='ignore'" >> ${_av_output_file}
        LDAPSERVER_CLIENT_FORCE_TLS='ignore'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_MONITOR_ENABLED}" -a -z "`grep ^LDAPSERVER_MONITOR_ENABLED ${source_conf}`" ]
    then
        echo "- LDAPSERVER_MONITOR_ENABLED='no'" >> ${_av_output_file}
        LDAPSERVER_MONITOR_ENABLED='no'
        _av_added=1
    fi

    # v0.90.2
    if [ -z "${LDAPSERVER_SCHEMA_CUSTOMIZE}" -a -z "`grep ^LDAPSERVER_SCHEMA_CUSTOMIZE ${source_conf}`" ]
    then
        echo "- LDAPSERVER_SCHEMA_CUSTOMIZE='no'" >> ${_av_output_file}
        LDAPSERVER_SCHEMA_CUSTOMIZE='no'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_SCHEMA_N}" -a -z "`grep ^LDAPSERVER_SCHEMA_N ${source_conf}`" ]
    then
        echo "- LDAPSERVER_SCHEMA_N='9'" >> ${_av_output_file}
        LDAPSERVER_SCHEMA_N='9'
        _av_added=1
    fi

    idx=1
    for def_sname in core cosine inetorgperson mozillaabpersonbeta nis postfix pureftpd resource samba3
    do
        if [ ${idx} -le ${LDAPSERVER_SCHEMA_N} ]
        then
            eval sname="\$LDAPSERVER_SCHEMA_${idx}_FILENAME"

            if [ -z "${sname}" -a -z "`grep ^LDAPSERVER_SCHEMA_${idx}_FILENAME ${source_conf}`" ]
            then
                echo "- LDAPSERVER_SCHEMA_${idx}_FILENAME='${def_sname}'" >> ${_av_output_file}
                eval "LDAPSERVER_SCHEMA_${idx}_FILENAME='${def_sname}'"
                _av_added=1
            fi
        fi

        idx=`expr ${idx} + 1`
    done

    # v1.4.0
    if [ -z "${LDAPSERVER_BACKUP}" -a -z "`grep ^LDAPSERVER_BACKUP ${source_conf}`" ]
    then
        echo "- LDAPSERVER_BACKUP='no'" >> ${_av_output_file}
        LDAPSERVER_BACKUP='no'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_BACKUP_PATH}" -a -z "`grep ^LDAPSERVER_BACKUP_PATH ${source_conf}`" ]
    then
        echo "- LDAPSERVER_BACKUP_PATH='/backup'" >> ${_av_output_file}
        LDAPSERVER_BACKUP_PATH='/backup'
        _av_added=1
    fi

    if [ -z "${LDAPSERVER_BACKUP_CRON_SCHEDULE}" -a -z "`grep ^LDAPSERVER_BACKUP_CRON_SCHEDULE ${source_conf}`" ]
    then
        echo "- LDAPSERVER_BACKUP_CRON_SCHEDULE='3 2 * * *'" >> ${_av_output_file}
        LDAPSERVER_BACKUP_CRON_SCHEDULE='3 2 * * *'
        _av_added=1
    fi

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "deleting old parameter(s) ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
    local _vs_var1
    local _vs_var2
    local _vs_ret

    eval _vs_var1=\$"${1}"
    eval _vs_var2=\$"{${1}+EMPTY}"

    if [ -z "${_vs_var1}" ]
    then
        if [ "${_vs_var2}" = 'EMPTY' ]
        then
            # variable is empty, but set
            _vs_ret=0
        else
            # variable is not set
            _vs_ret=1
        fi
    else
        # variable has a value
        _vs_ret=0
    fi

    return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "ldapserver" "2005-04-14" "jv" "Jens Vehlhaber"
        #------------------------------------------------------------------------------

        if [ "${LDAPSERVER_DO_DEBUG}" = "yes" -o "${LDAPSERVER_DO_DEBUG}" = "no" ]
        then
            # debug active
            printvar "LDAPSERVER_DO_DEBUG" "debug mode: yes or no"
            echo
        else
            # debug parameter exists but not active
            if [ -n "`grep LDAPSERVER_DO_DEBUG ${source_conf}`" ]
            then
                echo "# LDAPSERVER_DO_DEBUG='yes'                 # debug mode: yes or no"
                echo
            fi
        fi

        echo
        #------------------------------------------------------------------------------
        printgroup "General settings"
        #------------------------------------------------------------------------------
        printvar "START_LDAPSERVER"                "activate OpenLDAP: yes or no"
        echo
        printvar "LDAPSERVER_LDAP_BASEDC"          "base domain component"
        printvar "LDAPSERVER_LDAP_BASEDN"          "base distinguished name"
        printvar "LDAPSERVER_LDAP_ADMIN_NAME"      "name of LDAP administrator"
        printvar "LDAPSERVER_LDAP_ADMIN_PASS"      "password of LDAP administrator"

        #------------------------------------------------------------------------------
        printgroup "Security settings"
        #------------------------------------------------------------------------------
        printvar "LDAPSERVER_TRANSPORT"            "transport to use: default, tls, both"
        printvar "LDAPSERVER_TLS_PROTOCOL"         "tls protocol: none, auto, ssl2, ssl3, tls"
        printvar "LDAPSERVER_CLIENT_FORCE_TLS"     "force tls: yes, no (try), ignore"

        #------------------------------------------------------------------------------
        printgroup "other settings"
        #------------------------------------------------------------------------------
        printvar "LDAPSERVER_MONITOR_ENABLED"      "monitor ldap server: yes or no"

        #------------------------------------------------------------------------------
        printgroup "ldap schema settings"
        #------------------------------------------------------------------------------
        printvar "LDAPSERVER_SCHEMA_CUSTOMIZE"     "customize ldap schemas: yes or no"

        if [ -z "${LDAPSERVER_SCHEMA_N}" ]
        then
            LDAPSERVER_SCHEMA_N=0
        fi

        printsetvar "LDAPSERVER_SCHEMA_N"      "number of ldap schemas"

        imax=${LDAPSERVER_SCHEMA_N}

        idx=1
        while [ ${idx} -le ${imax} ]
        do
            printsetvar "LDAPSERVER_SCHEMA_${idx}_FILENAME" "${idx}. ldap schema"

            idx=`expr ${idx} + 1`
        done

        #------------------------------------------------------------------------------
        printgroup "ldap database backup settings"
        #------------------------------------------------------------------------------
        printvar "LDAPSERVER_BACKUP"               "activate database backup: yes or no"
        printvar "LDAPSERVER_BACKUP_PATH"          "directory path to store backups"
        printvar "LDAPSERVER_BACKUP_CRON_SCHEDULE" "cron configuration string"

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}

    # set access rights
    chmod 0644 ${generate_conf}
    chown root ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

basefile=/etc/config.d/base
configfile=/etc/config.d/ldapserver
installfile=/var/run/ldapserver.install
tmpfile=/var/run/ldapserver.$$

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case "$1" in
    update)
        goflag=1
        ;;
    test)
        source_conf=${configfile}

        generate_conf=/etc/config.d/capi2text.test
        goflag=1
        ;;
    *)
        echo
        echo "Use one of the following options:"
        echo
        echo "  ldapserver-update.sh [update] - the file ${configfile}.import will be read, the configuration will"
        echo "                                be checked and an updated mail configuration file will be written."
        echo
        goflag=0
        ;;
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists
        myecho --pack ${module_name} --info "previous configuration found ..."
        . ${basefile}
        . ${source_conf}

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
    fi

    if [ ${modflag} -eq 1 ]
    then
        myanykey --pack ${module_name}
    fi
fi

#==============================================================================
# end
#==============================================================================
exit 0
