#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/ldapserver.sh - configuration generator script for OpenLDAP
#
# Copyright (c) 2004-2008  Jens Vehlhaber <jvehlhaber(at)buchenwald(dot)de>
# Copyright (c) 2009-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2005-04-14 jv
# Last Update:  $Id: ldapserver.sh 73451 2020-03-24 13:19:47Z jed $
#
# Usage:        ldapserver.sh [--quiet] commands
#
# List of available commands:
#               ** no command ** - generate configuration files
#
#               --quiet          - suppress all screen output
#
#               --createtree     - remove existing ldap tree and create a new one
#               --upgradetree    - upgrade existing ldap tree
#               --removecron     - remove cronjob
#               --showsasl       - show supported SASL mechanisms
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

#include eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# check directory access
#------------------------------------------------------------------------------
check_directories ()
{
    # create config path
    if [ ! -d ${ldap_confdir} ]
    then
        mkdir -p ${ldap_confdir}
    fi

    if [ ! -d ${ldap_confdir} ]
    then
        mkdir -p ${ldap_confdir}/schema
    fi

    chown -R ${user}  ${ldap_confdir}
    chgrp -R ${group} ${ldap_confdir}
    chmod    0555     ${ldap_confdir}
    chmod -R 0550     ${ldap_confdir}/schema

    # create db path
    if [ ! -d ${ldap_datadir} ]
    then
        mkdir -p ${ldap_datadir}
    fi

    chown -R ${user}  ${ldap_datadir}
    chgrp -R ${group} ${ldap_datadir}
    chmod -R 0700     ${ldap_datadir}
}

#------------------------------------------------------------------------------
# create LDAP base configuration - ldap.base
#------------------------------------------------------------------------------
create_ldapbase_conf ()
{
    myecho --pack ${module_name} "Creating LDAP base configuration ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_ldapbase_file}" "${pgmname}" "ldapserver" "${ldapserver_version}"
        #-------------------------------------------------------------------------
        echo "OPENLDAP_DC_NAME='${LDAPSERVER_LDAP_BASEDC}'"
        echo "OPENLDAP_DC_BASE='${LDAPSERVER_LDAP_BASEDN}'"
        echo "OPENLDAP_CN_NAME='${LDAPSERVER_LDAP_ADMIN_NAME}'"
        echo "OPENLDAP_CN_PASS='`/usr/sbin/slappasswd -s ${LDAPSERVER_LDAP_ADMIN_PASS} -h {MD5}`'"
        echo
    } > ${generate_ldapbase_file}

    chown ${user}  ${generate_ldapbase_file}
    chgrp ${group} ${generate_ldapbase_file}
    chmod 0600     ${generate_ldapbase_file}
}

#------------------------------------------------------------------------------
# create LDAP server configuration - slapd.conf
#------------------------------------------------------------------------------
create_slapd_conf ()
{
    . ${generate_ldapbase_file}

    myecho --pack ${module_name} "Creating LDAP server configuration ..."

    if [ "${LDAPSERVER_MONITOR_ENABLED}" = 'yes' ]
    then
        myecho --pack ${module_name} "LDAP-Monitoring has been enabled."
    fi

    if [ -z "${LDAPSERVER_SCHEMA_N}" ]
    then
        LDAPSERVER_SCHEMA_N=0
    fi

    if [ "${LDAPSERVER_SCHEMA_CUSTOMIZE}" = "yes" -a ${LDAPSERVER_SCHEMA_N} -gt 0 ]
    then
        myecho --pack ${module_name} "Using customized LDAP schemas..."
    else
        myecho --pack ${module_name} "Using default LDAP schemas ..."
    fi

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_slapdconf_file}" "${pgmname}" "ldapserver" "${ldapserver_version}"
        #-------------------------------------------------------------------------

        echo '# Schema include'

        if [ "${LDAPSERVER_SCHEMA_CUSTOMIZE}" = "yes" -a ${LDAPSERVER_SCHEMA_N} -gt 0 ]
        then
            idx=1
            while [ ${idx} -le ${LDAPSERVER_SCHEMA_N} ]
            do
                eval ldap_schema_name="\$LDAPSERVER_SCHEMA_${idx}_FILENAME"

                echo "include ${ldap_confdir}/schema/${ldap_schema_name}.schema"

                idx=`expr ${idx} + 1`
            done
        else
            # default
            echo "include ${ldap_confdir}/schema/core.schema"
            echo "include ${ldap_confdir}/schema/cosine.schema"
            echo "include ${ldap_confdir}/schema/inetorgperson.schema"

          # JED - pimped the mozillaAbPersonAlpha.schema file by adding fields
          #       from the MoreFunctionsForAddressBook addin
          # echo "include ${ldap_confdir}/schema/mozillaabpersonalpha.schema"
            echo "include ${ldap_confdir}/schema/mozillaabpersonbeta.schema"

            echo "include ${ldap_confdir}/schema/nis.schema"
          # echo 'include ${ldap_confdir}/schema/rfc2307bis.schema"
            echo "include ${ldap_confdir}/schema/postfix.schema"
            echo "include ${ldap_confdir}/schema/pureftpd.schema"
            echo "include ${ldap_confdir}/schema/resource.schema"
          # echo "include ${ldap_confdir}/schema/samba2.schema"
            echo "include ${ldap_confdir}/schema/samba3.schema"
        fi

        echo
        echo '# Define global ACLs to disable default read access.'
        echo
        echo '# PID Files'
        echo "pidfile  ${pid_file}"
        echo 'argsfile /var/run/slapd.args'
        echo
        echo '# Default password type'
        echo 'password-hash {MD5}'
        echo
        echo '# Dynamic backend modules'
        echo
        echo '# Access control policy'
        echo 'access to attrs=userPassword'
        echo "  by dn=\"cn=${OPENLDAP_CN_NAME},${OPENLDAP_DC_BASE}\" write"
        echo '  by self write'
        echo '  by * auth'
        echo
        echo 'access to *'
        echo "  by dn=\"cn=${OPENLDAP_CN_NAME},${OPENLDAP_DC_BASE}\" write"
        echo '  by * read'
        echo
        echo '# Database definitions'
        echo 'database bdb'
        echo "suffix \"${OPENLDAP_DC_BASE}\""
        echo "rootdn \"cn=${OPENLDAP_CN_NAME},${OPENLDAP_DC_BASE}\""
        echo "rootpw ${OPENLDAP_CN_PASS}"
        echo
        echo '# Database directory'
        echo "directory ${ldap_datadir}"
        echo
        echo "# indices to maintain"
        echo "# PRESence, EQuality, SUBstring"

        # indices added due to sync provider support
        # source: http://www.openldap.org/doc/admin24/replication.html#Syncrepl
      # echo "index objectclass eq"
        echo "index objectClass pres,eq"
        echo "index entryCSN    eq"
        echo "index entryUUID   eq"

        echo "index cn          pres,sub,eq"
        echo "index sn          pres,sub,eq"
      # echo "index uid         eq"
        echo "index uid         pres,sub,eq"
        echo "index uidNumber   eq"
        echo "index gidNumber   eq"
        echo "index memberUID   eq"
        echo "index mail        sub,eq"
        echo "index default     sub"
        echo

        # indices added to speed-up phone number searches
        # field 'facsimileTelephoneNumber couldn't be indexed because it is of
        # type 'text string' and therefore sub-string searches are not allowed
        echo "index homePhone                sub,eq"
        echo "index telephoneNumber          sub,eq"
        echo "index mobile                   sub,eq"
        echo "index pager                    sub,eq"

        # indice added to speed-up name search
        echo "index givenName                sub,eq"
        echo

      # # act as a sync provider / JED
      # # source: http://www.openldap.org/doc/admin24/replication.html#Syncrepl
      # echo "# set-up synchronization provider"
      # echo "overlay syncprov"
      # echo "syncprov-checkpoint 100 10"
      # echo "syncprov-sessionlog 100"

      # # act as sync consumer / JED
      # echo "syncrepl rid=123"
      # echo "  provider=ldap://provider.example.com:389"
      # echo "  type=refreshOnly"
      # echo "  interval=01:00:00:00"
      # echo "  searchbase=\"dc=example,dc=com\""
      # echo "  filter=\"(objectClass=organizationalPerson)\""
      # echo "  scope=sub"
      # echo "  attrs=\"cn,sn,ou,telephoneNumber,title,l\""
      # echo "  schemachecking=off"
      # echo "  bindmethod=simple"
      # echo "  binddn=\"cn=syncuser,dc=example,dc=com\""
      # echo "  credentials=secret"

        if [ "${LDAPSERVER_MONITOR_ENABLED}" = "yes" ]
        then
            echo '# Monitor database definition'
            echo 'database monitor'
            echo 'access to dn.subtree="cn=Monitor"'
            echo "  by dn.exact=\"cn=${OPENLDAP_CN_NAME},${OPENLDAP_DC_BASE}\" read"
            echo '  by * none'
            echo
        fi

        echo "#allow bind_v2"

      # echo
        echo "# Loglevel"

        #   -1  enable all debugging
        #    0  no debugging
        #    1  trace function calls
        #    2  debug packet handling
        #    4  heavy trace debugging
        #    8  connection management
        #   16  print out packets sent and received
        #   32  search filter processing
        #   64  configuration file processing
        #  128  access control list processing
        #  256  stats log connections/operations/results (DEFAULT)
        #  512  stats log entries sent
        # 1024  print communication with shell backends
        # 2048  print entry parsing debugging

        if [ "${LDAPSERVER_DO_DEBUG}" = 'no' ]
        then
            echo 'loglevel 0'
        else
            echo 'loglevel 256'
        fi

        if [ "${LDAPSERVER_TRANSPORT}" = "tls" -o "${LDAPSERVER_TRANSPORT}" = "both" ]
        then
            # default, tls, both
            if [ -f ${sslcert_path}/slapd.pem -o -h ${sslcert_path}/slapd.pem ]
            then
                # certificate exists, go on ...
                echo
                echo '# CA signed certificate and server cert'

                case ${LDAPSERVER_TLS_PROTOCOL} in
                    auto)
                        echo 'TLSCipherSuite HIGH:MEDIUM'
                        ;;
                    ssl2)
                        echo 'TLSCipherSuite HIGH:MEDIUM:+SSLv2'
                        ;;
                    ssl3)
                        echo 'TLSCipherSuite HIGH:MEDIUM:+SSLv3'
                        ;;
                    tls)
                        echo 'TLSCipherSuite HIGH:MEDIUM:+TLSv1'
                        ;;
                    none)
                        ;;
                esac

              # echo "TLSCACertificateFile  ${sslcert_path}/ca.pem"
                echo "TLSCACertificatePath  ${sslcert_path}"
                echo "TLSCertificateFile    ${sslcert_path}/slapd.pem"
                echo "TLSCertificateKeyFile ${sslcert_path}/slapd.pem"
              # echo "TLSDHParamFile        ${sslcert_path}/slapd.pem"

                case ${LDAPSERVER_CLIENT_FORCE_TLS} in
                    yes)
                        # The client certificate is requested. If no certificate is
                        # provided, or a bad certificate is provided, the session is
                        # immediately terminated.
                        echo "TLSVerifyClient demand"
                        ;;
                    no)
                        # The client certificate is requested. If no certificate is
                        # provided, the session proceeds normally. If a bad certificatei
                        # is provided, the session is immediately terminated.
                        echo "TLSVerifyClient try"
                        ;;
                    ignore)
                        # default: slapd will not ask the client for a certificate
                        echo "TLSVerifyClient never"
                        ;;
                esac
            else
                echo "# error: slapd.pem certificate not found!"
            fi
        fi
    } > ${generate_slapdconf_file}

    chown ${user}  ${generate_slapdconf_file}
    chgrp ${group} ${generate_slapdconf_file}
    chmod 0600     ${generate_slapdconf_file}
}

#------------------------------------------------------------------------------
# update/check ldap.conf file
#------------------------------------------------------------------------------
update_ldap_conf ()
{
    if [ "${LDAPSERVER_TRANSPORT}" = "tls" -o "${LDAPSERVER_TRANSPORT}" = "both" ]
    then
        myecho --pack ${module_name} "Checking LDAP client configuration ..."

        if [ ! -f ${ldapconf_file} ]
        then
            # file does't exist, create it
            if [ -f ${ldapconf_file}.default ]
            then
                cp ${ldapconf_file}.default ${ldapconf_file}
            else
                # error
                myecho --pack ${module_name} --error "LDAP file '${ldapconf_file}.default' couldn't be found!"
            fi
        fi

        if [ -f ${ldapconf_file} ]
        then
            # make sure that the path to the certificates has been set
            grep -E -q "TLS_CACERTDIR" ${ldapconf_file}

            if [ $? -ne 0 ]
            then
                {
                    echo "# TLS parameters"
                    echo "TLS_CACERTDIR ${sslcert_path}"
                } >> ${ldapconf_file}
            fi
        else
            myecho --pack ${module_name} --warn "LDAP file '${ldapconf_file}' couldn't be found!"
        fi
    fi
}

#------------------------------------------------------------------------------
# create LDAP db configuration - DB_CONFIG
#------------------------------------------------------------------------------
create_dbconf_conf ()
{
    myecho --pack ${module_name} "Creating LDAP database configuration ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_dbconf_file}" "${pgmname}" "ldapserver" "${ldapserver_version}"
        #-------------------------------------------------------------------------

        # The cache size is defined by adding the Gbyte an byte value!
        # cache size in giga-bytes..:   0Gb                     -> 0
        # cache size in bytes.......: 256Mb (256 * 1024 * 1024) -> 268435456
        # number of caches to create:   1                       -> 1
        # Source: http://www.oracle.com/technology/documentation/berkeley-db/db/api_reference/C/dbset_cachesize.html
        echo "# set cache size to 256Mb"
        echo "set_cachesize 0 268435456 1"
        echo

        # The size of the logging area in the Berkeley DB environment, in bytes.
        # Source: http://www.oracle.com/technology/documentation/berkeley-db/db/api_reference/C/envset_lg_regionmax.html
        echo "# transaction log settings"
        echo "set_lg_regionmax 262144"

        # The size of the in-memory log buffer, in bytes.
        # Source: http://www.oracle.com/technology/documentation/berkeley-db/db/api_reference/C/envset_lg_bsize.html
        echo "set_lg_bsize     2097152"

        # The path of a directory to be used as the location of logging files.
        # Source: http://www.oracle.com/technology/documentation/berkeley-db/db/api_reference/C/envset_lg_dir.html
      # echo "set_lg_dir       logs"
      # echo
      # echo "# Note: special DB_CONFIG flags are no longer needed for 'quick'"
      # echo "# slapadd(8) or slapindex(8) access (see their -q option)."
    } > ${generate_dbconf_file}

    chown ${user}  ${generate_dbconf_file}
    chgrp ${group} ${generate_dbconf_file}
}

#------------------------------------------------------------------------------
# create temporary LDIF file - base.ldif
#------------------------------------------------------------------------------
create_tmp_ldif_file ()
{
    . ${basefile}
    . ${generate_ldapbase_file}

    myecho --pack ${module_name} "Creating temporary LDIF file ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_baseldif_file}" "${pgmname}" "openldap" "${openldap_version}"
        #-------------------------------------------------------------------------
        echo "# create base dn"
        echo "dn: ${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: dcObject"
        echo "objectClass: organization"
        echo "o: Eisfair ${OPENLDAP_DC_NAME}"
        echo "dc: ${OPENLDAP_DC_NAME}"
        echo
        echo "# create Directory Manager"
        echo "dn: cn=manager,${OPENLDAP_DC_BASE}"
        echo "cn: Manager"
        echo "objectClass: organizationalRole"
        echo "objectClass: top"
        echo "description: Directory Manager"
        echo "roleOccupant: cn=${OPENLDAP_CN_NAME},ou=users,${OPENLDAP_DC_BASE}"
        echo
        echo "# create groups"
        echo
        echo "dn: ou=machines,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: machines"
        echo "description: Machines - Computer"
        echo
        echo "dn: ou=groups,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: groups"
        echo "description: User groups"
        echo
        echo "dn: ou=transport,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: transport"
        echo "description: Mail-Transport"
        echo
        echo "dn: ou=users,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: users"
        echo "description: User group"
        echo
        echo "dn: ou=global-addressbook,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: global-addressbook"
        echo "description: Global addressbook"
        echo
        echo "# create object DNS"
        echo "dn: o=dns,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organization"
        echo "o: dns"
        echo
        echo "# create DNS domain"
        echo "dn: ou=${DOMAIN_NAME},o=dns,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: organizationalUnit"
        echo "ou: ${DOMAIN_NAME}"
        echo
        echo "# create localhost"
        echo "dn: cn=${HOSTNAME},ou=machines,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: device"
        echo "objectClass: ipHost"
        echo "ipHostNumber: ${IP_ETH_1_IPADDR}"
        echo "cn: ${HOSTNAME}"
        echo
        echo "# create group users"
        echo "dn: cn=users,ou=groups,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: posixGroup"
        echo "gidNumber: 100"
        echo 'userPassword: {crypt}*'
        echo "description: User group"
        echo "cn: users"
        echo
        echo "# create mailserver"
        echo "dn: cn=${DOMAIN_NAME},ou=transport,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: transportTable"
        echo "cn: ${DOMAIN_NAME}"
        echo "transport: smtp:[${IP_ETH_1_IPADDR}]"
        echo
        echo "# create demo entry for global addressbook"
        echo "dn: cn=User,ou=global-addressbook,${OPENLDAP_DC_BASE}"
        echo "objectClass: inetOrgPerson"
        echo "cn: User"
        echo "sn: Tester"
        echo "mail: user@virtual.test"
        echo
        echo "# create admin account"
        echo "dn: uid=${OPENLDAP_CN_NAME},ou=users,${OPENLDAP_DC_BASE}"
        echo "objectClass: top"
        echo "objectClass: posixAccount"
        echo "objectClass: shadowAccount"
        echo "objectClass: inetOrgPerson"
        echo "uidNumber: 999"
        echo "gidNumber: 100"
        echo "uid: ${OPENLDAP_CN_NAME}"
        echo "cn: ${OPENLDAP_CN_NAME}"
        echo "userPassword: ${OPENLDAP_CN_PASS}"
        echo "homeDirectory: /bin/false"
        echo "loginShell: /dev/null"
        echo "gecos: Adminuser ${OPENLDAP_CN_NAME}"
        echo "shadowFlag: 0"
        echo "shadowMin: 0"
        echo "shadowMax: 99999"
        echo "shadowWarning: 0"
        echo "shadowInactive: 99999"
        echo "shadowLastChange: 12011"
        echo "shadowExpire: 99999"
        echo "givenName: First name"
        echo "sn: ${OPENLDAP_CN_NAME}"
        echo "displayName: Admin ${OPENLDAP_CN_NAME}"
        echo
        echo "# --------------------------------------------------------"
        echo "# include all exists user"
        echo "# --------------------------------------------------------"
    } > ${generate_baseldif_file}

    old_ifs="${IFS}"
    IFS=":"

    # read passwd database (/etc/passwd)
    {
        getent passwd | while read pw_user pw_p2 pw_uid pw_gid pw_user_long pw_home pw_shell
        do
            # xxport only user with id >= 2000 and id < 65534
            if [ ${pw_uid} -ge 2000 -a ${pw_uid} -lt 65534 ]
            then
                echo "dn: uid=${pw_user},ou=users,${OPENLDAP_DC_BASE}"
                echo "objectClass: top"
                echo "objectClass: posixAccount"
                echo "objectClass: shadowAccount"
                echo "objectClass: inetOrgPerson"
                echo "objectClass: mozillaAbPersonAlpha"
                echo "uidNumber: ${pw_uid}"
                echo "gidNumber: ${pw_gid}"
                echo "uid: ${pw_user}"
                echo "cn: ${pw_user}"

                getent shadow | while read sh_user sh_pass sh_p3 sh_p4
                do
                    if [ "${sh_user}" = "${pw_user}"  ]
                    then
                        break
                    fi
                done

                echo "userPassword: {CRYPT}${sh_pass}"
                echo "homeDirectory: ${pw_home}"
                echo "loginShell: ${pw_shell}"
                echo "gecos: Eisfair user ${pw_user_long}"
                echo "shadowFlag: 0"
                echo "shadowMin: 0"
                echo "shadowMax: 99999"
                echo "shadowWarning: 0"
                echo "shadowInactive: 99999"
                echo "shadowLastChange: 12011"
                echo "shadowExpire: 99999"
                echo "givenName: First name"
                echo "sn: ${pw_user}"
                echo "displayName: ${pw_user_long}"
                echo
            fi
        done
    } >> ${generate_baseldif_file}

    IFS="${old_ifs}"

    chown ${user}  ${generate_baseldif_file}
    chgrp ${group} ${generate_baseldif_file}
}

#----------------------------------------------------------------------------------------
# add ldap service
#----------------------------------------------------------------------------------------
add_ldap_services ()
{
    myecho --pack ${module_name} "checking ldap/ldaps services ..."

    {
        echo "#"
        echo "# ${generate_services} list file generated by ${pgmname} ${ldapserver_version}"
        echo "#"

        # LDAP
        echo "ldap          389/tcp            # Lightweight Directory Access Protocol"
        echo "ldap          389/udp            # Lightweight Directory Access Protocol"

        # secure LDAP
        echo "ldaps         636/tcp            # LDAP over SSL"
        echo "ldaps         636/udp            # LDAP over SSL"
    } > ${generate_services}

    /var/install/bin/update-services ldapserver
}

#------------------------------------------------------------------------------
# update ldapserver configuration
#------------------------------------------------------------------------------
update_ldapserver_config ()
{
    cp ${configfile} ${configfile}.tmp

    {
        sed -e "/LDAPSERVER_LDAP_BASEDC/s/=[\"\'].*[\"\']/=\"${LDAPSERVER_LDAP_BASEDC}\"/g" \
            -e "/LDAPSERVER_LDAP_BASEDN/s/=[\"\'].*[\"\']/=\"${LDAPSERVER_LDAP_BASEDN}\"/g" \
            -e "/LDAPSERVER_LDAP_ADMIN_NAME/s/=[\"\'].*[\"\']/=\"${LDAPSERVER_LDAP_ADMIN_NAME}\"/g" $configfile.tmp
    } > ${configfile}

    chmod 600 ${configfile}

    rm -f ${configfile}.tmp
}

#------------------------------------------------------------------------------
# update admin password in slapd.conf
#------------------------------------------------------------------------------
update_slapd_password ()
{
    cp ${generate_slapdconf_file} ${generate_slapdconf_file}.tmp

    sed "/rootpw/s/rootpw .*$/rootpw ${OPENLDAP_CN_PASS}/g" ${generate_slapdconf_file}.tmp > ${generate_slapdconf_file}

    rm -f ${generate_slapdconf_file}.tmp

    chown ${user}  ${generate_slapdconf_file}
    chgrp ${group} ${generate_slapdconf_file}
}

#------------------------------------------------------------------------------
# wait until ldap daemon has been started
#------------------------------------------------------------------------------
wait_for_start_ldap_daemon ()
{
    _wfs_ret=1

    # wait <= 10s for ldap start ldap
    count=0
    while [ ${count} -lt 11 ]
    do
        myecho --pack ${module_name} -n "."

        count=`expr ${count} + 1`
        sleep 1

        if [ -e ${pid_file} ]
        then
            # file exists
            break
        fi
    done

    if [ ${count} -eq 11 ]
    then
        myecho --pack ${module_name} " failed!"
    elif [ ${count} -ge 1 ]
    then
        _wfs_ret=0
        myecho --pack ${module_name} " started."
    fi

    return ${_wfs_ret}
}

#------------------------------------------------------------------------------
# show supported SASL mechanisms
#------------------------------------------------------------------------------
show_sasl_mechanisms ()
{
    . ${clientconfigfile}

    if [ "${START_LDAPCLIENT}" = 'yes' ]
    then
        # ldap client package has been activated
        _ssm_mechs=`ldapsearch -LLL -x -H ${LDAPCLIENT_LDAP_HOSTURI} -s "base" -b "" supportedSASLMechanisms | \
                        sed -e '/^dn:/d' -e '/^supportedSASLMechanisms:/s/^supportedSASLMechanisms: *//g' | \
                        tr '\n' ' ' | sed 's/ *$//'`

        myecho --pack ${module_name} "supported SASL mechanisms: ${_ssm_mechs}"
    else
        myecho --pack ${module_name} --warn "unable to display the available SASL mechanisms,"
        myecho --pack ${module_name} --warn "because the ldapclient package hasn't been enabled."
    fi
}

#------------------------------------------------------------------------------
# add cron job
#------------------------------------------------------------------------------
add_cron_job ()
{
    myecho --pack ${module_name} "creating cron job ..."

    # check for cron directory
    if [ ! -d ${crontab_path} ]
    then
        mkdir -p ${crontab_path}
    fi

    # write file
    {
        echo "#--------------------------------------------------------------------"
        echo "#  cron file generated by ${pgmname} version: ${ldapserver_version}"
        echo "#"
        echo "#  Do not edit this file, edit ${configfile}"
        echo "#  Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#--------------------------------------------------------------------"
        echo "${LDAPSERVER_BACKUP_CRON_SCHEDULE} /var/install/bin/ldapserver-backup-database -backuppath ${LDAPSERVER_BACKUP_PATH}"
        echo
    } > ${crontab_file}

    # update crontab file
    /var/install/config.d/cron > /dev/null 2> /dev/null
}

#------------------------------------------------------------------------------
# delete cron job
#------------------------------------------------------------------------------
delete_cron_job ()
{
    myecho --pack ${module_name} "deleting cron job ..."

    # check for crontab file
    if [ -f ${crontab_file} ]
    then
        # delete existing file
        rm -f ${crontab_file}

        # update crontab file
        /var/install/config.d/cron > /dev/null 2> /dev/null
    fi
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

### set path names ###
ldap_confdir=/etc/openldap
ldap_datadir=/var/lib/openldap
sslcert_path=/usr/local/ssl/certs
crontab_path=/var/cron/etc/root

### set file names ###
basefile=/etc/config.d/base
configfile=/etc/config.d/ldapserver
clientconfigfile=/etc/config.d/ldapclient
crontab_file=${crontab_path}/ldapserver
generate_services=/etc/services.ldapserver
generate_baseldif_file=${ldap_datadir}/base.ldif
generate_ldapbase_file=${ldap_datadir}/ldap.base
generate_dbconf_file=${ldap_datadir}/DB_CONFIG
generate_slapdconf_file=${ldap_confdir}/slapd.conf
ldapconf_file=${ldap_confdir}/ldap.conf
pid_file=/var/run/slapd.pid

user='ldap'
group='ldap'

### other parameters ###
if [ "`/usr/bin/eisman check ${module_name}`" = 'installed' ]
then
    ldapserver_version="`/usr/bin/eisman query --installed --fields=version --no-tty ${module_name} | sed -e '/^version/!d' -e 's/^version *//' | tail -n1`"
else
    ldapserver_version='unknown'
fi

. ${configfile}

force_quiet_run=false
if [ $# -gt 0 ]
then
    # read parameter(s)
    while [ $# -gt 0 ]
    do
        case $1 in
            *-quiet )
                force_quiet_run=true
                shift
                ;;
            * )
                cmd="$1"
                break
                ;;
        esac
    done
fi

case ${cmd} in
    *-removecron|removecron )
        delete_cron_job
        ;;
    *-showsasl|showsasl )
        show_sasl_mechanisms
        ;;
    *-createtree|createtree|*-upgradetree|upgradetree )
        create_flag=0
        case ${cmd} in
            *-createtree|createtree )
                create_flag=1
                ;;
        esac

        if [ ${create_flag} -eq 1 -a -f ${generate_ldapbase_file} ]
        then
            echo
            myecho --pack ${module_name} --force-output --warn "OpenLDAP tree already exists"
            if ! /var/install/bin/ask "Delete old tree?" "no"
            then
                exit 0
            fi
        fi

        # create new ldap tree
        if [ -f ${pid_file} ]
        then
            /etc/init.d/ldapserver stop
        fi

        rm -f ${ldap_datadir}/*

        # dialog - create new ldap tree
        clrhome

        if [ ${create_flag} -eq 1 ]
        then
            myecho --pack ${module_name} --force-output --info "Create new LDAP tree"
            myecho --pack ${module_name} --force-output ""
            myecho --pack ${module_name} --force-output "Please enter the following information:"
            myecho --pack ${module_name} --force-output ""

            LDAPSERVER_LDAP_BASEDC=`/var/install/bin/ask "First base name...:" "${LDAPSERVER_LDAP_BASEDC}" "+"`
            myecho --pack ${module_name} --force-output ""

            LDAPSERVER_LDAP_BASEDN=`/var/install/bin/ask "Second base name..:" "${LDAPSERVER_LDAP_BASEDN}" "+"`
            myecho --pack ${module_name} --force-output ""

            LDAPSERVER_LDAP_ADMIN_NAME=`/var/install/bin/ask "Administrator name:" "${LDAPSERVER_LDAP_ADMIN_NAME}" "+"`
            myecho --pack ${module_name} --force-output ""

            ldif_file=''
            while [ -z "${ldif_file}" ]
            do
                ldif_file=`/var/install/bin/ask "Enter initial LDIF file name:" "default" "+"`

                if [ "${ldif_file}" != "default" ]
                then
                    echo ${ldif_file} | grep -q "\.ldif$"

                    if [ $? -ne 0 ]
                    then
                        ldif_file=${ldif_file}.ldif
                    fi

                    if [ ! -f ${ldif_file} ]
                    then
                        myecho --pack ${module_name} --force-output --error "LDIF file '${ldif_file}' doesn't exist!"
                        ldif_file=''
                    fi
                fi
            done

            myecho --pack ${module_name} ""
        else
            myecho --pack ${module_name} --info "Update LDAP tree"
        fi

        myecho --pack ${module_name} "${module_name} version: ${ldapserver_version}"

        create_ldapbase_conf

        create_dbconf_conf
        create_slapd_conf
        update_ldap_conf
        update_ldapserver_config

        /etc/init.d/ldapserver start

        myecho --pack ${module_name} -n "Starting LDAP server "

        if wait_for_start_ldap_daemon
        then
            if [ ${create_flag} -eq 1 ]
            then
                if [ "${ldif_file}" = "default" ]
                then
                    # clean installation
                    create_tmp_ldif_file
                else
                    # custom installation
                    cp ${ldif_file} ${generate_baseldif_file}
                fi

                myecho --pack ${module_name} "Creating initial LDAP tree ..."
            else
                # upgrade database from old openldap-package
                cp /root/openldap-export.ldif ${generate_baseldif_file}
                mv /root/openldap-export.ldif /root/openldap-export.ldif.OLD

                myecho --pack ${module_name} "Updating LDAP tree ..."
            fi

            sleep 1

            # -h host    LDAP server
            # -f file    read operations from `file'
            # -x         Simple authentication
            # -D binddn  bind DN
            # -w passwd  bind password (for simple authentication)
            ldapadd -f ${generate_baseldif_file} -x -D "cn=${OPENLDAP_CN_NAME},${OPENLDAP_DC_BASE}" -w ${LDAPSERVER_LDAP_ADMIN_PASS} 1>/dev/null

            if [ $? -eq 0 ]
            then
                myecho --pack ${module_name} "Removing temporary LDIF file ..."
                rm -f ${generate_baseldif_file}
            fi
        else
            myecho --pack ${module_name} --force-output --error "unable to start LDAP server, configuration aborted!"
        fi
        ;;
    * )
        if [ "${START_LDAPSERVER}" = "yes" ]
        then
            check_directories
            add_ldap_services

            if [ ! -f ${generate_ldapbase_file} ]
            then
                $0 --createtree
            elif [ -f /root/openldap-export.ldif ]
            then
                $0 --upgradetree
            else
                # generate all configuration files
                myecho --pack ${module_name} "${module_name} version: ${ldapserver_version}"

                # change ldap parameter
                create_ldapbase_conf

                create_dbconf_conf
                create_slapd_conf
                update_ldap_conf

                add_cron_job

                myecho --pack ${module_name} "finished."
            fi
        else
            myecho --pack ${module_name} "${module_name} version: ${ldapserver_version}"
            myecho --pack ${module_name} --force-output --warn "ldapserver is currently disabled!"
            delete_cron_job
        fi
        ;;
esac

#==============================================================================
# end
#==============================================================================
exit 0
